/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.imp;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.filechooser.FileView;
import org.netbeans.api.project.Project;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.filesystems.FileChooserBuilder;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.Lookup;
import org.openide.util.NbBundle.Messages;

@ActionID(category = "File",
id = "nl.cloudfarming.client.imp.ImportAction")
@ActionRegistration(iconBase = "nl/cloudfarming/client/imp/import.png",
displayName = "#CTL_ImportAction")
@ActionReferences({
    @ActionReference(path = "Menu/File", position = 0)
})
@Messages({"CTL_ImportAction=Import", "ImportAction chooser title=Import file"})
public final class ImportAction extends AbstractAction {

    private static final boolean FILE_HIDING = true;
    private final Project context;
    private final static Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.imp.ImportAction");
    private final int STARTING_LEVEL = 0;
    private final int MAX_LEVEL = 2;

    public ImportAction(Project context) {
        super(Bundle.ImportAction_chooser_title());
        this.context = context;
    }

    @Override
    public void actionPerformed(ActionEvent ev) {
        FileChooserBuilder builder = new FileChooserBuilder(ImportAction.class);
        builder.setTitle(Bundle.ImportAction_chooser_title());
        builder.setFileHiding(FILE_HIDING);

        Collection<? extends ImportProcessor> importProcessors = Lookup.getDefault().lookupAll(ImportProcessor.class);

        //If only one found set the file filter as primairy
        if (importProcessors.size() == 1) {
            ImportProcessor importProcessor = importProcessors.iterator().next();
            FileFilter filter = new FileNameExtensionFilter(importProcessor.getFormatDescription(), importProcessor.getSupportedExtentions());
            builder.setFileFilter(filter);
        } else {
            //else add all the file filters
            for (ImportProcessor importProcessor : importProcessors) {
                FileFilter filter = new FileNameExtensionFilter(importProcessor.getFormatDescription(), importProcessor.getSupportedExtentions());
                builder.addFileFilter(filter);
            }
        }
        final JFileChooser chooser = builder.createFileChooser();
        chooser.setMultiSelectionEnabled(true);
        chooser.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (JFileChooser.APPROVE_SELECTION.equals(e.getActionCommand())) {
                    for (File file : chooser.getSelectedFiles()) {
                        importFile(file, STARTING_LEVEL);
                    }
                }
            }
        });
        chooser.setFileView(new FileView() {

            @Override
            public Icon getIcon(File f) {
                Icon icon = super.getIcon(f);
                FileObject fileObject = FileUtil.toFileObject(f);
                if (fileObject != null) {
                    for (ImportProcessor importProcessor : Lookup.getDefault().lookupAll(ImportProcessor.class)) {
                        if (importProcessor.isSupported(fileObject)) {
                            icon = importProcessor.getIcon();
                            break;
                        }
                    }
                }
                return icon;
            }
        });
        chooser.showOpenDialog(null);


    }

    /**
     * Imports the files If the level becomes above 1 stops recursively adding
     * files to prevent out of memory
     *
     * @param file
     * @param level
     */
    protected void importFile(File file, int level) {
        LOGGER.log(Level.FINEST, "starting import for selected file {0}", file.getAbsolutePath());
        // recurse if file is a folder
        if (file.isDirectory()) {
            for (File child : file.listFiles()) {
                if (level < MAX_LEVEL) {
                    importFile(child, level + 1);
                }
            }
        } else {
            // search for available processor
            for (ImportProcessor importProcessor : Lookup.getDefault().lookupAll(ImportProcessor.class)) {
                FileObject fo = FileUtil.toFileObject(file);
                if (fo != null && importProcessor.isSupported(fo)) {
                    LOGGER.log(Level.FINEST, "Processor found for file {0}", importProcessor.toString());
                    importProcessor.doImport(fo, context);
                    break;
                }
            }
        }
    }
}
