/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.imp;

import java.io.IOException;
import java.util.List;
import javax.swing.Icon;
import javax.swing.SwingUtilities;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.nodes.Node;

/**
 * Capability of importing fileobject into AgroSense.
 *
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public interface ImportProcessor {

    public static final String IMPORT_FOLDER = "Import";
    /**
     * Array with supported extentions, is shown in open file dialog
     *
     * @return
     */
    String[] getSupportedExtentions();

    /**
     * Return true if file format is supported for processing by this processor
     * - Check Extension - Check Fileheader
     *
     * @param sensorFile
     * @return
     */
    boolean isSupported(FileObject importFile);

    /**
     * localized description of supported formats
     *
     * for example:
     *
     * "Cropcircle data (*.csv; *.txt)"
     *
     * @return
     */
    String getFormatDescription();
    
    /**
     * Get the icon representing this type of file.
     * <p>
     * Will be used in the generic import file picker to display the proper
     * icon for recognized filetypes. This makes it easier for the user
     * to pick the proper file.
     * </p>
     * <p>
     * Users of this method should be aware to only use icons for supported file
     * types ie invoke {@link ImportProcessor#isSupported(org.openide.filesystems.FileObject) }
     * before this method.
     * </p>
     * 
     * @return 
     */
    Icon getIcon();

    /**
     * Import files associated with specified file object to the proper import folder
     * <p>
     * Implementations can use {@link Util#getImportFolder(org.netbeans.api.project.Project) }
     * to obtain the projects root import folder.
     * </p>
     * @param importFile
     * @param project
     * @throws IllegalArgumentException
     */
    void doImport(FileObject importFile, Project project) throws IllegalArgumentException;

    /**
     * process the content of an importfile <p> Content of the file will be
     * processed and mapped to entities available for the provided farm project.
     *
     * for example process on a shape file can result in aditional field and/or
     * treatment zones. </p> <p> This method can initiate a wizard or other gui
     * panel. In this case the gui event will have to be started by a {@link SwingUtilities#invokeAndWait(java.lang.Runnable)
     * }
     * or {@link SwingUtilities#invokeLater(java.lang.Runnable) }
     * After the user interaction the process will have to finish. </p>
     *
     * @param importFile file object to import
     * @param project to import the data to
     * @throws IllegalArgumentException when importFile is not supported.
     */
    void process(FileObject importFile, Project project) throws IllegalArgumentException;

    /**
     * Same as {@link ImportProcessor#process(org.openide.filesystems.FileObject, org.netbeans.api.project.Project)
     * }
     * but with additional argument to map processed data to.
     *
     * @see ImportProcessor#process(org.openide.filesystems.FileObject,
     * org.netbeans.api.project.Project)
     * @param importFile file object to import
     * @param project to import the data to
     * @param toPopulate List of provided type to be filled by the process
     * routine.
     * @throws IllegalArgumentException
     */
    <T extends Object> void process(FileObject importFile, Project project, List<T> toPopulate) throws IllegalArgumentException;
    
    
    
    /**
     * Get the node representing the processor specific sub folder.
     * <p>
     * will use {@link ImportProcessor.Util#getImportFolder(org.netbeans.api.project.Project) } to determine 
     * the import folder for the specified project.
     * </p>
     * <p>
     * Implementations should typically return a FilterNode for the processor
     * specific DataFolder node within the projects import folder.
     * </p>
     * <p>
     * Result will be used to display imported content.
     * Implementation can add actions for controlling the content, or an open
     * action to open an overview of the content.
     * </p>
     * 
     * @param project 
     * @return processor specific import node
     */
    public Node getImportNode(Project project);

    static class Util {
        
        public static FileObject getImportFolder(Project project) throws IOException {
            FileObject folder = project.getProjectDirectory().getFileObject(IMPORT_FOLDER);
            if (folder == null) {
                folder = project.getProjectDirectory().createFolder(IMPORT_FOLDER);
            }
            return folder;
        }
    }
}
